library(MASS)
### Boston dataset
?Boston
names(Boston)
View(Boston)

############################### LINEAR REGGRESION #######################################

### Linear reggresion
lm.fit=lm(medv~lstat,data=Boston)
lm.fit
summary(lm.fit)
confint(lm.fit, level = 0.95) #confidence intervals for model parametrs
predict(lm.fit, interval="confidence") #confidence for given values

attach(Boston)
plot(lstat,medv)
abline(lm.fit,lwd=3,col="red")

par(mfrow=c(2,2))
plot(lm.fit)

###Non-linear transformations of the predictors
orderedBoston <- Boston[order(Boston$lstat),]
lm.fit4=lm(medv~ lstat + I(lstat^2),data=orderedBoston) #linear reggresion
lm.fit4
summary(lm.fit4)
par(mfrow=c(2,2))
plot(lm.fit4)
dev.off()

plot(orderedBoston$lstat, orderedBoston$medv, xlab = "lstat", ylab = "medv", main = "Linear vs Non-linear regression curve")
lines(orderedBoston$lstat, predict(lm.fit4), col="red", lwd=3)
abline(lm.fit, col = "blue", lwd=3)
legend("topright", col=c("red", "blue"), c("Non-linear curve", "Linear curve"), lty = 1, lwd = 3)


###Linear reggresion with two predictors
lm.fit2=lm(medv~lstat+rm,data=Boston)
lm.fit2
summary(lm.fit2)
par(mfrow=c(2,2))
plot(lm.fit2)


###Linear reggresion with all predictors
lm.fit3=lm(medv~.,data=Boston)
lm.fit3
summary(lm.fit3)
par(mfrow=c(2,2))
plot(lm.fit3)

############################### SUBSET SELECTIONS METHODS #######################################
library('leaps')
regfit.full=regsubsets(medv~., data = Boston)
summary(regfit.full)
regfit.full=regsubsets(medv~., data = Boston,nvmax=19)
reg.summary=summary(regfit.full)
names(reg.summary)

#which model to select
names(reg.summary)
reg.summary$rsq
par(mfrow=c(2,2))
plot(reg.summary$rss,xlab="Number of Variables",ylab="RSS",type="l")
plot(reg.summary$adjr2,xlab="Number of Variables",ylab="Adjusted RSq",type="l") #l parametr plotted points with lines
which.max(reg.summary$adjr2)
points(11,reg.summary$adjr2[11], col="red",cex=2,pch=20)
plot(reg.summary$cp,xlab="Number of Variables",ylab="Cp",type='l')
which.min(reg.summary$cp)
points(11,reg.summary$cp[11],col="red",cex=2,pch=20)
which.min(reg.summary$bic)
plot(reg.summary$bic,xlab="Number of Variables",ylab="BIC",type='l')
points(11,reg.summary$bic[11],col="red",cex=2,pch=20)

plot(regfit.full,scale="r2")
plot(regfit.full,scale="adjr2")
plot(regfit.full,scale="Cp")
plot(regfit.full,scale="bic")
coef(regfit.full,11)



###################### PRCINCIPAL COMPONENT ANALYSIS #######################
apply(Boston, 2, mean)
apply(Boston, 2, var)
pr.out=prcomp(Boston, scale=TRUE, center = TRUE)
names(pr.out)
pr.out$center
pr.out$scale
pr.out$rotation
dim(pr.out$x)
biplot(pr.out, scale=0)
pr.out$rotation=-pr.out$rotation
pr.out$x=-pr.out$x
biplot(pr.out, scale=0, choices = c(1,3))
pr.out$sdev
pr.var=pr.out$sdev^2
pr.var
pve=pr.var/sum(pr.var)
pve
par(mfrow=c(1,2))
plot(pve, xlab="Principal Component", ylab="Proportion of Variance Explained", ylim=c(0,1),type='b')
plot(cumsum(pve), xlab="Principal Component", ylab="Cumulative Proportion of Variance Explained", ylim=c(0,1),type='b')

### K-means clustering
set.seed(2)
pr.matrix <- pr.out$x[,c(1,3)]

km.out=kmeans(pr.matrix,2,nstart=20)
km.out$cluster
plot(pr.matrix, col=(km.out$cluster+1), main="K-Means Clustering Results with K=2", xlab="", ylab="", pch=20, cex=2)
set.seed(4)
km.out=kmeans(pr.matrix,3,nstart=20)
km.out
plot(pr.matrix, col=(km.out$cluster+1), main="K-Means Clustering Results with K=3", xlab="", ylab="", pch=20, cex=2)
set.seed(3)
km.out=kmeans(pr.matrix,4,nstart=20)
km.out
plot(pr.matrix, col=(km.out$cluster+1), main="K-Means Clustering Results with K=3", xlab="", ylab="", pch=20, cex=2)


### Hierarchical clustering
hc.complete=hclust(dist(pr.matrix), method="complete")
hc.average=hclust(dist(pr.matrix), method="average")
hc.single=hclust(dist(pr.matrix), method="single")
par(mfrow=c(1,3))
plot(hc.complete,main="Complete Linkage", xlab="", sub="", cex=.9)
plot(hc.average, main="Average Linkage", xlab="", sub="", cex=.9)
plot(hc.single, main="Single Linkage", xlab="", sub="", cex=.9)
cutree(hc.complete, 2)
cutree(hc.average, 2)
cutree(hc.single, 2)
cutree(hc.single, 4)
